

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.docx4j.model.structure.HeaderFooterPolicy;
import org.docx4j.model.structure.SectionWrapper;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.io.SaveToZipFile;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.CustomXmlDataStoragePart;
import org.docx4j.openpackaging.parts.CustomXmlDataStoragePropertiesPart;
import org.docx4j.openpackaging.parts.DocPropsCorePart;
import org.docx4j.openpackaging.parts.DocPropsCustomPart;
import org.docx4j.openpackaging.parts.DocPropsExtendedPart;
import org.docx4j.openpackaging.parts.Part;
import org.docx4j.openpackaging.parts.WordprocessingML.HeaderPart;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
import org.docx4j.relationships.Relationship;

import org.docx4j.wml.CTRel;
import org.docx4j.wml.FooterReference;
import org.docx4j.wml.Hdr;
import org.docx4j.wml.HdrFtrRef;
import org.docx4j.wml.HeaderReference;
import org.docx4j.wml.ObjectFactory;
import org.docx4j.wml.SectPr;

public class HeaderFooterTest {
    ObjectFactory objFac = new ObjectFactory();
    public HeaderFooterTest() {
        super();
    }
    
    public WordprocessingMLPackage getMainDoc(String filename){
        WordprocessingMLPackage wordMLPackage = null;
        try {
            wordMLPackage = WordprocessingMLPackage.load(new java.io.File(filename));
        } catch (Docx4JException e) {
            System.out.println(e.getMessage());
        }
        return wordMLPackage;
    }
    
    public void createHeaderFooterReference(WordprocessingMLPackage wordprocessingMLPackage, Relationship headerRelationship,int headerType) throws InvalidFormatException {
        List<SectionWrapper> sections = wordprocessingMLPackage.getDocumentModel().getSections(); 
        SectPr sectPr = sections.get(sections.size() - 1).getSectPr();
        // There is always a section wrapper, but it might not contain a sectPr
        if (sectPr==null ) {
            sectPr = objFac.createSectPr();
            wordprocessingMLPackage.getMainDocumentPart().addObject(sectPr);
            sections.get(sections.size() - 1).setSectPr(sectPr);
        }
        HeaderReference headerReference = objFac.createHeaderReference();
        headerReference.setId(headerRelationship.getId());
        if(headerType == 0) {
            headerReference.setType(HdrFtrRef.DEFAULT);
            System.out.println("Addition of default");
        }
        if(headerType == 1) {
            headerReference.setType(HdrFtrRef.EVEN);
            System.out.println("Addition of even");
        }
        //FooterReference footerReference = objFac.createFooterReference();
        //footerReference.setId(footerRelationship.getId());
        //footerReference.setType(HdrFtrRef.DEFAULT);
        sectPr.getEGHdrFtrReferences().add(headerReference);
        System.out.println("Size of sectPR list: "+sectPr.getEGHdrFtrReferences().size());
        //sectPr.getEGHdrFtrReferences().add(footerReference);
    }
    
    public void removeRelationships(HeaderPart p){
        if (p.getRelationshipsPart()!= null ) {
           RelationshipsPart rp=p.getRelationshipsPart();
           List<Relationship> deletions = new ArrayList<Relationship>();
           for ( Relationship r : rp.getRelationships().getRelationship() )
              deletions.add(r);
           for ( Relationship r : deletions) {
              rp.removeRelationship(r);
            }    
        }
    }
    
    public void removeRelationships(DocPropsCustomPart p){
        if (p.getRelationshipsPart()!= null ) {
           RelationshipsPart rp=p.getRelationshipsPart();
           List<Relationship> deletions = new ArrayList<Relationship>();
           for ( Relationship r : rp.getRelationships().getRelationship() )
              deletions.add(r);
           for ( Relationship r : deletions) {
              rp.removeRelationship(r);
            }    
        }     
    }
    
    public void removeRelationships(CustomXmlDataStoragePart p){
        if (p.getRelationshipsPart()!= null ) {
           RelationshipsPart rp=p.getRelationshipsPart();
           List<Relationship> deletions = new ArrayList<Relationship>();
           for ( Relationship r : rp.getRelationships().getRelationship() )
              deletions.add(r);
           for ( Relationship r : deletions) {
              rp.removeRelationship(r);
            }    
        }        
    }
    
    public void removeHeaderFooterReferences(WordprocessingMLPackage wordprocessingMLPackage){
        List<SectionWrapper> sections = wordprocessingMLPackage.getDocumentModel().getSections(); 
        HeaderPart header = null,eHeader = null;
        for (SectionWrapper sw : sections) {
            header = sw.getHeaderFooterPolicy().getDefaultHeader();
            if(header != null){
                removeRelationships(header);
                wordprocessingMLPackage.getMainDocumentPart().getRelationshipsPart().removeRelationship(header.getPartName());
            }
            //eHeader = sw.getHeaderFooterPolicy().getEvenHeader();
            //if(eHeader != null){
                //removeRelationships(eHeader);
                //wordprocessingMLPackage.getMainDocumentPart().getRelationshipsPart().removeRelationship(eHeader.getPartName());
            //}
                List<CTRel> rel =sw.getSectPr().getEGHdrFtrReferences();
                Iterator<CTRel> relIter = rel.iterator();
                while(relIter.hasNext()){
                    CTRel ctRel = relIter.next();
                    if (ctRel instanceof HeaderReference) {
                        HeaderReference hr = (HeaderReference) ctRel;
                        if (hr.getType().equals(HdrFtrRef.DEFAULT)||hr.getType().equals(HdrFtrRef.EVEN)) {
                            //sw.getSectPr().getEGHdrFtrReferences().remove(hr);
                            relIter.remove();
                            System.out.println("removal!");
                        }
                    }
                }
            System.out.println("Size of sectPr after removal: "+sw.getSectPr().getEGHdrFtrReferences().size());
        }
    }
    
    public void addSubParts(DocPropsCustomPart mainPart,MainDocumentPart mainDocumentPart){
        RelationshipsPart rp = mainPart.getRelationshipsPart(false);
        if(rp == null){
          //there are no relationships => no sub parts to add
         return;
        //if there is a sub part add this to the part
        } else{      
          try{
            for(Relationship r : rp.getRelationships().getRelationship() ) {
              //get the part the relationship belongs to
                Part part = rp.getPart(r);      
                part.setPackage(mainDocumentPart.getPackage());    
                Relationship rh1 = mainDocumentPart.addTargetPart(part);
              //mainDocumentPart.getPart(mainPart.getPartName()).addTargetPart(part);
                //addSubParts(part,mainDocumentPart);
            }        
          } catch (InvalidFormatException e) {
            System.out.println(e.getMessage());
          }  
        }    
    }
    
    public void addSubParts(HeaderPart mainPart,MainDocumentPart mainDocumentPart){
        //get the relationships part of the document part
        //where the information is stored whether the part
        //has sub parts or not
        RelationshipsPart rp = mainPart.getRelationshipsPart(false);
        if(rp == null){
          //there are no relationships => no sub parts to add
         return;
        //if there is a sub part add this to the part
        } else{      
          try{
            for(Relationship r : rp.getRelationships().getRelationship() ) {
              //get the part the relationship belongs to
                Part part = rp.getPart(r);      
                part.setPackage(mainDocumentPart.getPackage());
                Relationship rh1 = mainDocumentPart.addTargetPart(part);
              //mainDocumentPart.getPart(mainPart.getPartName()).addTargetPart(part);
                //addSubParts(part,mainDocumentPart);
            }        
          } catch (InvalidFormatException e) {
            System.out.println(e.getMessage());
          }  
        }        
    }
   
    public void addSubParts(HeaderPart mainPart,HeaderPart newHeaderPart){
        //get the relationships part of the document part
        //where the information is stored whether the part
        //has sub parts or not
        RelationshipsPart rp = mainPart.getRelationshipsPart(false);
        if(rp == null){
          //there are no relationships => no sub parts to add
         return;
        //if there is a sub part add this to the part
        } else{      
          try{
            for(Relationship r : rp.getRelationships().getRelationship() ) {
              //get the part the relationship belongs to
                Part part = rp.getPart(r);      
                //part.setPackage(mainDocumentPart.getPackage());
                Relationship rh1 = newHeaderPart.addTargetPart(part);
              //mainDocumentPart.getPart(mainPart.getPartName()).addTargetPart(part);
                //addSubParts(part,mainDocumentPart);
            }        
          } catch (InvalidFormatException e) {
            System.out.println(e.getMessage());
          }  
        }        
    }
    
    public void addSubParts(Part mainPart,MainDocumentPart mainDocumentPart){
        RelationshipsPart rp = mainPart.getRelationshipsPart(false);
        if(rp == null){
          //there are no relationships => no sub parts to add
         return;
        //if there is a sub part add this to the part
        } else{      
          try{
            for(Relationship r : rp.getRelationships().getRelationship() ) {
              //get the part the relationship belongs to
                Part part = rp.getPart(r);      
                part.setPackage(mainDocumentPart.getPackage()); 
                Relationship rh1 = mainDocumentPart.addTargetPart(part);
              //mainDocumentPart.getPart(mainPart.getPartName()).addTargetPart(part);
                addSubParts(part,mainDocumentPart);
            }        
          } catch (InvalidFormatException e) {
            System.out.println(e.getMessage());
          }  
        }    
    }
    
    public void headerMethod() throws InvalidFormatException {
        String templatePath = "C:\\Users\\prsrikan.SYMPHONY\\AppData\\Roaming\\Microsoft\\Templates\\oddEven2.dotx";
        WordprocessingMLPackage wordMLPackage = getMainDoc(templatePath);
        DocPropsCorePart coreDoc = wordMLPackage.getDocPropsCorePart();
        DocPropsCustomPart customDoc = wordMLPackage.getDocPropsCustomPart();
        DocPropsExtendedPart appDoc = wordMLPackage.getDocPropsExtendedPart();
        //to do: custom xml copy
        HashMap<String,CustomXmlDataStoragePart> customXMLdoc = wordMLPackage.getCustomXmlDataStorageParts();
        //target document
        String existingFileName = "D:/NewEmptyDoc.docx";
        WordprocessingMLPackage wordMLPackage1 = getMainDoc(existingFileName);
        MainDocumentPart mainDocumentPart1 = wordMLPackage1.getMainDocumentPart();
        DocPropsCorePart coreDoc1 = wordMLPackage1.getDocPropsCorePart();
        DocPropsCustomPart customDoc1 = wordMLPackage1.getDocPropsCustomPart();
        DocPropsExtendedPart appDoc1 = wordMLPackage1.getDocPropsExtendedPart();
        HashMap<String,CustomXmlDataStoragePart> customXMLdoc1 = wordMLPackage1.getCustomXmlDataStorageParts();
        //removing header references of existing doc
        removeHeaderFooterReferences(wordMLPackage1);
        List<SectionWrapper> sections = wordMLPackage.getDocumentModel().getSections();
        for(SectionWrapper sw : sections){
            HeaderFooterPolicy hfp = sw.getHeaderFooterPolicy();
            HeaderPart header = hfp.getDefaultHeader();
        
            HeaderPart newHeader = null;
            if(header!=null){
                Hdr hdr = header.getJaxbElement();
                newHeader = new HeaderPart();
                newHeader.setJaxbElement(hdr);
                newHeader.setPackage(wordMLPackage1);
                Relationship relShip = null;
                relShip = mainDocumentPart1.addTargetPart(newHeader);
                createHeaderFooterReference(wordMLPackage1,relShip,0);
                //addSubParts(header,mainDocumentPart1);
                addSubParts(header,newHeader);
            }
            HeaderPart eHeader = hfp.getEvenHeader();
            if(eHeader!=null){
                Hdr hdr = eHeader.getJaxbElement();
                newHeader = new HeaderPart();
                newHeader.setJaxbElement(hdr);
                newHeader.setPackage(wordMLPackage1);
                Relationship relShip = null;
                relShip = mainDocumentPart1.addTargetPart(newHeader);
                createHeaderFooterReference(wordMLPackage1,relShip,0);
                addSubParts(eHeader,newHeader);  
            }
        }
        coreDoc1.setJaxbElement(coreDoc.getJaxbElement());
        appDoc1.setJaxbElement(appDoc.getJaxbElement());
        //custom part of docProps
        if(customDoc!=null){
            if(customDoc1 != null){
                removeRelationships(customDoc1);
            }
            customDoc1 = new DocPropsCustomPart();
            customDoc1.setJaxbElement(customDoc.getJaxbElement());
            customDoc1.setPackage(wordMLPackage1);
            //Relationship relShip = mainDocumentPart1.addTargetPart(customDoc1);
            //addSubParts(customDoc,mainDocumentPart1);
        }
        //custom XML part
        if(customXMLdoc.size() != 0){
            if(customXMLdoc1.size() != 0){
                System.out.println(customXMLdoc1.size());
                for(CustomXmlDataStoragePart custPart : customXMLdoc1.values()){
                    removeRelationships(custPart);             
                }
                System.out.println(customXMLdoc1.size());
                customXMLdoc1.clear();
                System.out.println("CustomXML existence");
            }
            System.out.println("CustomXML manipulation");
            for(String key : customXMLdoc.keySet()){
                CustomXmlDataStoragePart newCustPart = new CustomXmlDataStoragePart();
                CustomXmlDataStoragePart oldCustPart = customXMLdoc.get(key);
                newCustPart.setPackage(wordMLPackage1);
                newCustPart.setData(oldCustPart.getData());
                RelationshipsPart rp = oldCustPart.getRelationshipsPart(false);
                for(Relationship r : rp.getRelationships().getRelationship()){
                    Part pr = rp.getPart(r);
                    if(pr instanceof CustomXmlDataStoragePropertiesPart){
                        newCustPart.addTargetPart(pr);
                        break;
                    }
                }
                mainDocumentPart1.addTargetPart(newCustPart);
                customXMLdoc1.put(key, newCustPart);
            }       
        }
        SaveToZipFile saver = new SaveToZipFile(wordMLPackage1);
        try {
            saver.save(existingFileName);
        } catch (Docx4JException e) {
            System.out.println(e.getMessage());
        }
        for (SectionWrapper sw : sections) {
            HeaderFooterPolicy hfp = sw.getHeaderFooterPolicy();
            System.out.println("\n\nSECTION  \n");
            System.out.println("Headers:");
            if (hfp.getFirstHeader()!=null) System.out.println("-first"); 
                if (hfp.getDefaultHeader()!=null) System.out.println("-default"); 
                    if (hfp.getEvenHeader()!=null) System.out.println("-even");                    
        }
        System.out.println("**********************");
        List<SectionWrapper> sectionWrappers = wordMLPackage1.getDocumentModel().getSections();                
        for (SectionWrapper sw : sectionWrappers) {
            HeaderFooterPolicy hfp = sw.getHeaderFooterPolicy();
            System.out.println("\n\nSECTION  \n");
            System.out.println("Headers:");
            if (hfp.getFirstHeader()!=null) System.out.println("-first"); 
                if (hfp.getDefaultHeader()!=null) System.out.println("-default"); 
                    if (hfp.getEvenHeader()!=null) System.out.println("-even");                    
        }
    }    
}
